@@ -1,5 +1,5 @@
 /* Distributed under the OSI-approved BSD 3-Clause License.  See accompanying
-   file Copyright.txt or https://cmake.org/licensing for details.  */
+   file LICENSE.rst or https://cmake.org/licensing for details.  */
 
 #include "cmDyndepCollation.h"
 
@@ -239,7 +239,7 @@ Json::Value CollationInformationBmiInstallation(cmGeneratorTarget const* gt,
 
     tdi_bmi_info["permissions"] = bmi_gen->GetFilePermissions();
     tdi_bmi_info["destination"] = bmi_gen->GetDestination(config);
-    const char* msg_level = "";
+    char const* msg_level = "";
     switch (bmi_gen->GetMessageLevel()) {
       case cmInstallGenerator::MessageDefault:
         break;
@@ -644,6 +644,9 @@ bool cmDyndepCollation::WriteDyndepMetadata(
             result = false;
           }
         }
+        for (auto const& req : object.Requires) {
+          bdb_entry->second->Requires.push_back(req.LogicalName);
+        }
       } else if (export_info.DatabaseInfo) {
         cmSystemTools::Error(cmStrCat(
           "Failed to find module database entry for ", file_set.SourcePath));
